/* vim: set ts=4: */
/*
 * Copyright (C) 2008 Daniel Morales <daniel@daniel.com.uy>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gstdio.h>
#include <gtk/gtkbuilder.h>

#include <thunar-vfs/thunar-vfs.h>

#include "tsp-page.h"
#include "shares.h"

#define G_STR_EMPTY(x) ((x) == NULL || (x)[0] == '\0')

/* Property identifiers */
enum
{
	PROP_0,
	PROP_FILE,
};

static void tsp_page_class_init    (TspPageClass    *klass);
static void tsp_page_init          (TspPage         *tsp_page);
static void tsp_page_finalize      (GObject         *object);
static void tsp_page_get_property  (GObject         *object,
                                    guint            prop_id,
                                    GValue          *value,
                                    GParamSpec      *pspec);
static void tsp_page_set_property  (GObject         *object,
                                    guint            prop_id,
                                    const GValue    *value,
                                    GParamSpec      *pspec);
static void tsp_page_file_changed  (ThunarxFileInfo *file,
                                    TspPage         *tsp_page);
static void tsp_page_share_toggled (GtkToggleButton *togglebutton,
                                    TspPage         *tsp_page);
static void tsp_page_apply_clicked (GtkButton       *button,
                                    TspPage         *tsp_page);
static void tsp_page_sensibility   (TspPage         *tsp_page,
                                    gboolean         sens);
static void tsp_show_error         (TspPage         *tsp_page,
                                    const char      *msg);

static gboolean tsp_check_perms    (TspPage         *tsp_page,
                                    ThunarVfsInfo   *file,
                                    const gchar     *path,
                                    gboolean         is_writable);

static gboolean tsp_ask_perms      (TspPage         *tsp_page,
                                    gboolean         need_r,
                                    gboolean         need_w,
                                    gboolean         need_x);

static GtkBuilder *tsp_get_xml     (void);

struct _TspPageClass
{
	ThunarxPropertyPageClass __parent__;
};

struct _TspPage
{
	ThunarxPropertyPage __parent__;
	ThunarxFileInfo    *file;

	GtkWidget          *cb_share_folder;
	GtkWidget          *entry_share_name;
	GtkWidget          *cb_share_write;
	GtkWidget          *entry_share_comments;
	GtkWidget          *label_comments;
	GtkWidget          *label_name;
	GtkWidget          *button_apply;
	GtkWidget          *cb_share_guest;
	GtkWidget          *label_status;	
};

/* implements the tsp_page_get_type() and tsp_page_register_type() functions */
THUNARX_DEFINE_TYPE (TspPage, tsp_page, THUNARX_TYPE_PROPERTY_PAGE);

static void
tsp_page_class_init (TspPageClass *klass)
{
	GObjectClass *gobject_class;

	gobject_class = G_OBJECT_CLASS (klass);
	gobject_class->finalize = tsp_page_finalize;
	gobject_class->get_property = tsp_page_get_property;
	gobject_class->set_property = tsp_page_set_property;

	/**
	* TspPage:file:
	*
	* The ThunarxFileInfo displayed by this TspPage.
	**/
	g_object_class_install_property (gobject_class,
                                     PROP_FILE,
                                     g_param_spec_object ("file", "file", "file",
                                                          THUNARX_TYPE_FILE_INFO,
                                                          G_PARAM_READWRITE));
}

static void
tsp_page_init (TspPage *tsp_page)
{
	GtkWidget *vbox;
	GtkBuilder *ui;
	
	ui = tsp_get_xml ();
	if (!ui)
		return;

	/* Get the widgets */
	tsp_page->cb_share_folder = GTK_WIDGET (gtk_builder_get_object (ui, "cb_share_folder"));
	tsp_page->label_name = GTK_WIDGET (gtk_builder_get_object (ui, "label_name"));
	tsp_page->entry_share_name = GTK_WIDGET (gtk_builder_get_object (ui, "entry_share_name"));
	tsp_page->cb_share_write = GTK_WIDGET (gtk_builder_get_object (ui, "cb_share_write"));
	tsp_page->label_comments = GTK_WIDGET (gtk_builder_get_object (ui, "label_comments"));
	tsp_page->entry_share_comments = GTK_WIDGET (gtk_builder_get_object (ui, "entry_share_comments"));
	tsp_page->button_apply = GTK_WIDGET (gtk_builder_get_object (ui, "button_apply"));
	tsp_page->cb_share_guest = GTK_WIDGET (gtk_builder_get_object (ui, "cb_share_guest"));
	tsp_page->label_status = GTK_WIDGET (gtk_builder_get_object (ui, "label_status"));

	/* Connect signals */
	g_signal_connect (tsp_page->cb_share_folder, "toggled", G_CALLBACK (tsp_page_share_toggled), tsp_page);
	g_signal_connect (tsp_page->button_apply, "clicked", G_CALLBACK (tsp_page_apply_clicked), tsp_page);

	vbox = GTK_WIDGET (gtk_builder_get_object (ui, "vbox1"));
	gtk_container_add (GTK_CONTAINER (tsp_page), vbox);
	gtk_container_set_border_width (GTK_CONTAINER (tsp_page), 5);
	gtk_widget_show_all (vbox);

	g_object_unref (ui);
}

static void
tsp_page_finalize (GObject *object)
{
	TspPage *tsp_page = TSP_PAGE (object);

	/* disconnect from the file */
	tsp_page_set_file (tsp_page, NULL);

	/* release any additional resources... */

	(*G_OBJECT_CLASS (tsp_page_parent_class)->finalize) (object);
}

static void
tsp_page_get_property (GObject    *object,
                       guint       prop_id,
                       GValue     *value,
                       GParamSpec *pspec)
{
	TspPage *tsp_page = TSP_PAGE (object);

	switch (prop_id)
	{
		case PROP_FILE:
			g_value_set_object (value, tsp_page_get_file (tsp_page));
		break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
tsp_page_set_property (GObject      *object,
                       guint         prop_id,
                       const GValue *value,
                       GParamSpec   *pspec)
{
	TspPage *tsp_page = TSP_PAGE (object);

	switch (prop_id)
	{
		case PROP_FILE:
			tsp_page_set_file (tsp_page, g_value_get_object (value));
		break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
		break;
	}
}

static void
tsp_page_file_changed (ThunarxFileInfo *file,
                       TspPage         *tsp_page)
{
	gchar *uri, *file_local;
	ShareInfo *share_info;
	gboolean result;

	/* Load share info */
	uri = thunarx_file_info_get_uri (tsp_page->file);
	file_local = g_filename_from_uri (uri, NULL, NULL);

	result = shares_get_share_info_for_path (file_local, &share_info, NULL);
	if (share_info)
	{
		tsp_page_sensibility (tsp_page, TRUE);

		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tsp_page->cb_share_folder), TRUE);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tsp_page->cb_share_write), share_info->is_writable);
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tsp_page->cb_share_guest), share_info->guest_ok);
		gtk_entry_set_text (GTK_ENTRY (tsp_page->entry_share_name), share_info->share_name);
		gtk_entry_set_text (GTK_ENTRY (tsp_page->entry_share_comments), share_info->comment);

		shares_free_share_info (share_info);
	} else {
		tsp_page_sensibility (tsp_page, FALSE);
	}

	g_free (uri);
	g_free (file_local);
}

static void
tsp_page_share_toggled (GtkToggleButton *togglebutton,
                        TspPage         *tsp_page)
{
	gboolean share = gtk_toggle_button_get_active (togglebutton);

	/* Toggle sharing */
	tsp_page_sensibility (tsp_page, share);

	/* Sharing (?) */
	if (share)
	{
		gchar *name_utf8;
		gchar *name;

		name = thunarx_file_info_get_name (tsp_page->file);
		name_utf8 = g_filename_display_name (name);

		/* First sharename is the filename */
		gtk_entry_set_text (GTK_ENTRY (tsp_page->entry_share_name), name_utf8);
		
		g_free (name_utf8);
		g_free (name);
	}
}

static void
tsp_page_apply_clicked (GtkButton *button,
                        TspPage   *tsp_page)
{
	ThunarVfsInfo *file_info;
	GError *err = NULL;
	gchar *file_local;
	gboolean share;
	gboolean ret;
	gchar *file;

	share = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tsp_page->cb_share_folder));

	file = thunarx_file_info_get_uri (tsp_page->file);
	file_local = g_filename_from_uri (file, NULL, NULL);

	if (share){
		ShareInfo *share_info;
		const gchar *comments;
		const gchar *name;
		gboolean writable;
		gboolean guest_ok;

		/* Check if we have a name for the share */
		name = gtk_entry_get_text (GTK_ENTRY (tsp_page->entry_share_name));
		if (G_STR_EMPTY (name)){
			tsp_show_error (tsp_page,
					_("Please, write a name for the share."));
		} else {
			comments = gtk_entry_get_text (GTK_ENTRY (tsp_page->entry_share_comments));
			writable = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tsp_page->cb_share_write));
			guest_ok = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tsp_page->cb_share_guest));

			/* TODO: check if the share name isn't used (?) */

			/* Check permissions */
			file_info = thunarx_file_info_get_vfs_info (tsp_page->file);
			if (tsp_check_perms (tsp_page, file_info, file_local, writable))
			{
				share_info = g_new0 (ShareInfo, 1);

				/* Fill the struct */
				share_info->path = g_strdup (file_local);
				share_info->share_name = g_strdup (name);
				if (G_STR_EMPTY (comments)){
					share_info->comment = g_strdup ("");
				} else {
					share_info->comment = g_strdup (comments);
				}
				share_info->is_writable = writable;
				share_info->guest_ok = guest_ok;

				/* Share it */
				ret = shares_modify_share (file_local, share_info, &err);
				if (!ret){
					tsp_show_error (tsp_page, err->message);
					g_error_free (err);
				}

				/* Free some memory */
				shares_free_share_info (share_info);
			} else {
				tsp_show_error (tsp_page,
						_("Insufficient permissions to share the folder."));
			}
			thunar_vfs_info_unref (file_info);
		}
	} else {
		gboolean is_shared;
		/* Check if this path is shared */
		shares_get_path_is_shared (file_local, &is_shared, NULL);
		if (is_shared)
		{
			/* Un-share it */
			ret = shares_modify_share (file_local, NULL, &err);
			if (!ret){
				tsp_show_error (tsp_page, err->message);
				g_error_free (err);
			}
		}
	}

	g_free (file_local);
	g_free (file);
}

static void
tsp_page_sensibility  (TspPage  *tsp_page,
                       gboolean  sens)
{
	gtk_widget_set_sensitive (GTK_WIDGET (tsp_page->entry_share_name), sens);
	gtk_widget_set_sensitive (GTK_WIDGET (tsp_page->cb_share_write), sens);
	gtk_widget_set_sensitive (GTK_WIDGET (tsp_page->entry_share_comments), sens);
	gtk_widget_set_sensitive (GTK_WIDGET (tsp_page->label_name), sens);
	gtk_widget_set_sensitive (GTK_WIDGET (tsp_page->label_comments), sens);
	gtk_widget_set_sensitive (GTK_WIDGET (tsp_page->label_comments), sens);
	gtk_widget_set_sensitive (GTK_WIDGET (tsp_page->cb_share_guest), sens);
}

GtkWidget*
tsp_page_new (ThunarxFileInfo *file)
{
	TspPage *page = g_object_new (TSP_TYPE_PAGE, "file", file, NULL);

	thunarx_property_page_set_label (THUNARX_PROPERTY_PAGE (page), "Share");

	return GTK_WIDGET (page);
}

ThunarxFileInfo*
tsp_page_get_file (TspPage *tsp_page)
{
	g_return_val_if_fail (TSP_IS_PAGE (tsp_page), NULL);
	return tsp_page->file;
}

void
tsp_page_set_file (TspPage         *tsp_page,
                   ThunarxFileInfo *file)
{
	g_return_if_fail (TSP_IS_PAGE (tsp_page));
	g_return_if_fail (file == NULL || THUNARX_IS_FILE_INFO (file));

	if (tsp_page->file != NULL)
	{
		g_signal_handlers_disconnect_by_func (tsp_page->file, tsp_page_file_changed, tsp_page);
		g_object_unref (G_OBJECT (tsp_page->file));
	}

	tsp_page->file = file;

	if (file != NULL)
	{
		g_object_ref (file);
		tsp_page_file_changed (file, tsp_page);
		g_signal_connect (file, "changed", G_CALLBACK (tsp_page_file_changed), tsp_page);
	}

	g_object_notify (G_OBJECT (tsp_page), "file");
}

static gboolean
tsp_check_perms (TspPage       *tsp_page,
                 ThunarVfsInfo *file,
                 const gchar   *path,
                 gboolean       is_writable)
{
	ThunarVfsFileMode new_mode;
	gboolean need_r;
	gboolean need_w;
	gboolean need_x;

	new_mode = file->mode;

	need_r = (file->mode & THUNAR_VFS_FILE_MODE_OTH_READ) == 0;
	new_mode |= THUNAR_VFS_FILE_MODE_OTH_READ;

	need_w = is_writable && (file->mode & THUNAR_VFS_FILE_MODE_OTH_WRITE) == 0;
	if (need_w)
		new_mode |= THUNAR_VFS_FILE_MODE_OTH_WRITE;

	need_x = (file->mode & THUNAR_VFS_FILE_MODE_OTH_EXEC) == 0;
	new_mode |= THUNAR_VFS_FILE_MODE_OTH_EXEC;

	if (need_r || need_w || need_x)
	{
		if (!tsp_ask_perms (tsp_page, need_r, need_w, need_x))
			return FALSE;

		g_message ("Changing permissions of '%s'", path);

		if (g_chmod (path, new_mode) != 0)
		{
			return FALSE;
		}
    }
	return TRUE;
}

static gboolean
tsp_ask_perms (TspPage  *tsp_page,
               gboolean  need_r,
               gboolean  need_w,
               gboolean  need_x)
{
	GtkWidget *dialog;
	gboolean result;
	GtkBuilder *ui;

	ui = tsp_get_xml ();
	if (!ui)
		return FALSE;

	dialog = GTK_WIDGET (gtk_builder_get_object (ui, "permissions_dialog"));

	result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK;

	gtk_widget_destroy (dialog);
	g_object_unref (ui);

	return result;
}

static GtkBuilder *
tsp_get_xml (void)
{
	GError *err = NULL;
	GtkBuilder *ui;
	gchar *path;

	ui = gtk_builder_new ();
	gtk_builder_set_translation_domain (ui, GETTEXT_PACKAGE);
	path = g_build_filename (DATADIR, "thunar-shares", "thunar-shares.xml", NULL);

	if (gtk_builder_add_from_file (ui, path, &err) == 0)
	{
		g_warning ("XML file error: %s", err->message);
		g_error_free (err);
		g_free (path);
		return NULL;
	}

	g_free (path);
	return ui;
}

static void
tsp_show_error (TspPage    *tsp_page,
                const char *msg)
{
	GtkWidget *dialog;
	GtkBuilder *ui;

	ui = tsp_get_xml ();
	if (!ui)
		return;

	dialog = GTK_WIDGET (gtk_builder_get_object (ui, "error_dialog"));
	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), msg);

	gtk_dialog_run (GTK_DIALOG (dialog));

	gtk_widget_destroy (dialog);
	g_object_unref (ui);
}
